@@ -52,8 +52,6 @@ def add_forecast_init(actions):
         help="Number of days used for model initialization. "
         "Defaults to 15 days spinup.",
         type=float, default=15.)
-    init.add_argument("--forecast-interval", type=float, default=24)
-    init.add_argument("--timezone", default='UTC')
     init.add_argument(
         "--skip-run", action="store_true",
         help="Skips running the model.")
@@ -64,9 +62,9 @@ def add_forecast_init(actions):
     # TODO: Additional forcings.
     # _add_wave_forcing(forecast)
     model_outputs = init.add_argument_group('model_outputs')
-    _add_surface_outputs(model_outputs)
     # TODO: Stations outputs.
-    # _add_stations_outputs(model_outputs)
+    _add_stations_outputs(model_outputs)
+    _add_surface_outputs(model_outputs)
     server_config = init.add_subparsers(dest="server_config")
     slurm = server_config.add_parser(
         'slurm', help="Add options for slurm run configuration.")
@@ -100,9 +98,6 @@ def add_forecast(subparsers):
     forecast.add_argument(
         "--overwrite", action="store_true",
         help="Allow overwrite of output directory.")
-    forecast.add_argument(
-        "--log-level",
-        choices=[name.lower() for name in logging._nameToLevel])
     actions = forecast.add_subparsers(dest="action")
     add_forecast_init(actions)
     add_forecast_update(actions)
@@ -183,8 +178,11 @@ def _add_tidal_constituents(parser):
         dest='constituents',
         default=False,
         help="Tidal constituent to be forced in the model (case-insensitive).")
+    tides.add_argument("--tidal-database", choices=['tpxo', 'hamtide'],
+                       default='tpxo')
     tides.add_argument("--include-tidal-velocity",
                        "--bnd-vel", action="store_true", dest='bnd_vel')
+    tides.add_argument("--Z0", type=float)
 
 
 def _add_atmospheric_forcing(parser):
@@ -616,3 +614,18 @@ def _add_surface_outputs(parser):
                 help=help_msg,
                 action='store_true'
             )
+
+
+def _add_stations_outputs(parser):
+    parser.add_argument('--stations-file')
+    parser.add_argument('--stations-file-crs')
+    parser.add_argument('--nspool-sta')
+    parser.add_argument('--stations-elev', action='store_true')
+    parser.add_argument('--stations-prmsl', action='store_true')
+    parser.add_argument('--stations-uwind', action='store_true')
+    parser.add_argument('--stations-vwind', action='store_true')
+    parser.add_argument('--stations-temp', action='store_true')
+    parser.add_argument('--stations-sal', action='store_true')
+    parser.add_argument('--stations-uvel', action='store_true')
+    parser.add_argument('--stations-vvel', action='store_true')
+    parser.add_argument('--stations-wvel', action='store_true')
