@@ -4,12 +4,15 @@ from enum import Enum
 from functools import lru_cache
 import logging
 
+import numpy as np
+
 from pyschism.domain import ModelDomain
 from pyschism.forcing.tides.tides import Tides
 from pyschism.param.param import Param
 
 _logger = logging.getLogger(__name__)
 
+
 class NullWritter:
 
     def __init__(self, *argv):
@@ -153,8 +156,9 @@ class Bctides:
                      f'{forcing[2]:G} ' \
                      f'{forcing[3]:G} ' \
                      f'{forcing[4]:G}\n'
-        f += f'{self.tidal_forcing.nbfr:d}\n'
-        for constituent in self.get_active_forcing_constituents():
+        afc = self._model_domain.get_active_forcing_constituents()
+        f += f'{len(afc):d}\n'
+        for constituent in afc:
             forcing = self.tidal_forcing(
                 self.start_date, self.rnday, constituent)
             f += f'{constituent} \n' \
